package be.iminds.ilabt.jfed.git;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;

/**
 * GitFetcherCommand: contains all info about:
 *   - what git repo
 *   - what data to get from it
 *   - where to put it
 */
public class GitFetcherCommand {
    @Nonnull private final String gitUrl;
    @Nullable private final String repoSubDir;
    @Nullable private final String branch;

    @Nullable private final GitAuthPreferences.GitAuth gitAuth;

    @Nullable private final File targetDir;
    private final boolean allowCreateDir;
    private final boolean overwriteFilesInDir;

    public GitFetcherCommand(@Nonnull  String gitUrl,
                             @Nullable String repoSubDir,
                             @Nullable String branch,
                             @Nullable GitAuthPreferences.GitAuth gitAuth,
                             @Nullable File targetDir,
                             boolean allowCreateDir,
                             boolean overwriteFilesInDir) {
        this.gitUrl = gitUrl;
        this.repoSubDir = repoSubDir;
        this.branch = branch;
        this.gitAuth = gitAuth;
        this.targetDir = targetDir;
        this.allowCreateDir = allowCreateDir;
        this.overwriteFilesInDir = overwriteFilesInDir;
    }

    public static File createTempDirectory() {
        try {
            return Files.createTempDirectory("git-repo").toFile();
        } catch (IOException e) {
            throw new RuntimeException("Could not create temporary dir for git repo files.", e);
        }
    }


//    public static Optional<GitFetcherCommand> fromString(@Nonnull String command) {
//        return fromString(command, null);
//    }
//
//    /**
//     * @return The GitFetcherCommand in the string, unless the string does not include one.
//     */
//    public static Optional<GitFetcherCommand> fromString(@Nonnull String command,
//                                                         @Nullable File targetDir) {
//        if (targetDir == null)
//            targetDir = createTempDirectory();
//
//        Optional<GitFetcherCommandBuilder> resBuilderOpt =
//                GitFetcherCommandBuilder.fromString(command);
//
//        if (!resBuilderOpt.isPresent())
//            return Optional.empty();
//
//        GitFetcherCommandBuilder resBuilder = resBuilderOpt.get();
//        resBuilder.setTargetDir(targetDir);
//
//        return Optional.of(resBuilder.build());
//    }

    @Nonnull
    public String getGitUrl() {
        return gitUrl;
    }

    @Nullable
    public String getRepoSubDir() {
        return repoSubDir;
    }

    @Nullable
    public String getBranch() {
        return branch;
    }

    @Nullable
    public GitAuthPreferences.GitAuth getGitAuth() {
        return gitAuth;
    }

    @Nullable
    public File getTargetDir() {
        return targetDir;
    }

    public boolean isAllowCreateDir() {
        return allowCreateDir;
    }

    public boolean isOverwriteFilesInDir() {
        return overwriteFilesInDir;
    }
}
